@@ -41,10 +41,11 @@ burdenMeta <- function(..., SNPInfo=NULL, wts = 1, snpNames = "Name", aggregateB
 			cohort.gene <- cohorts[[cohort.k]][[gene]]
 			
 			if(!is.null(cohort.gene)){
+				cohort.gene <- lapply(cohort.gene,function(x){replace(x,is.nan(x),0)})
 				sub <- match(snp.names.list[[gene]],colnames(cohort.gene$cov))
 				if(any(is.na(sub)) | any(sub != 1:length(sub), na.rm=TRUE) | length(cohort.gene$maf) > nsnps.sub){
 						cohort.gene$maf <- cohort.gene$maf[sub]
-						cohort.gene$maf[is.na(sub)] <- 0
+						cohort.gene$maf[is.na(sub)] <- -1
 						
 						cohort.gene$cov <- cohort.gene$cov[sub,sub,drop=FALSE]
 						cohort.gene$cov[is.na(sub),] <- cohort.gene$cov[,is.na(sub)] <- 0
@@ -57,7 +58,6 @@ burdenMeta <- function(..., SNPInfo=NULL, wts = 1, snpNames = "Name", aggregateB
 				n.total[cohort.gene$maf >= 0] <- n.total[cohort.gene$maf >= 0]+cohort.gene$n
 				cohort.gene$maf[cohort.gene$maf < 0] <- 0
 				
-				
 				maf <- maf + 2*cohort.gene$maf*cohort.gene$n
 				mscores <- mscores + cohort.gene$scores/cohort.gene$sey^2
 				big.cov <- big.cov + cohort.gene$cov/cohort.gene$sey^2
@@ -70,7 +70,7 @@ burdenMeta <- function(..., SNPInfo=NULL, wts = 1, snpNames = "Name", aggregateB
 		maf <- sapply(maf, function(x){min(x,1-x)})
 		
 		if(is.function(wts)){
-			tmpwts <- wts(maf)
+			tmpwts <- ifelse(maf > 0, wts(maf),0)
 		} else if(is.character(wts)){
 			tmpwts <- as.numeric(SNPInfo[SNPInfo[,aggregateBy]==gene,wts])
 		} else {
@@ -78,23 +78,22 @@ burdenMeta <- function(..., SNPInfo=NULL, wts = 1, snpNames = "Name", aggregateB
 		}
 		
 		if( !all(mafRange == c(0,0.5))){
-			keep <- (maf >= min(mafRange)) & (maf <= max(mafRange))
+			keep <- (maf > min(mafRange)) & (maf <= max(mafRange))
 			tmpwts[!keep] <- 0
 		}
 		
-		
 		vary.ave <- vary.ave/max(n.total,na.rm=TRUE)
 		bscorevar <- c(tmpwts%*%big.cov%*%tmpwts)
 		bscore <- tmpwts%*%mscores 
 		
-		res.numeric[ri,"beta"] = bscore/bscorevar
+		res.numeric[ri,"beta"] = ifelse(bscorevar !=0, bscore/bscorevar, NA)
 		res.numeric[ri,"se"] = sqrt(1/bscorevar)
 		res.numeric[ri,"cmafTotal"] = sum(maf,na.rm=TRUE)
 		res.numeric[ri,"cmafUsed"] = sum(maf[tmpwts != 0],na.rm=TRUE)
 		res.numeric[ri,"nsnpsTotal"] = length(maf)
 		res.numeric[ri,"nmiss"] = sum(n.miss[tmpwts != 0], na.rm =T)
 		res.numeric[ri,"nsnpsUsed"] = sum(tmpwts != 0)
-		res.numeric[ri,"p"] = pchisq(bscore^2/bscorevar,lower.tail=FALSE,df=1)
+		res.numeric[ri,"p"] = ifelse(bscorevar !=0,pchisq(bscore^2/bscorevar,lower.tail=FALSE,df=1),NA)
 		if(verbose){
 			pb.i <- pb.i+1
 			setTxtProgressBar(pb, pb.i)
